@@ -38,7 +38,7 @@ type configuration struct {
 
 func closeAndExit(client *xmpp.Client, err error) {
 	slog.Info("closing connection and exiting")
-	client.Close()
+	_ = client.Close()
 	if err != nil {
 		log.Fatal(err)
 	}
@@ -186,6 +186,8 @@ func main() {
 	case len(*flagHTTPUpload) != 0 && *flagMessageFile != "":
 		log.Fatal("You can't send a message while using" +
 			" http upload.")
+	case *flagOx && *flagLegacyPGP:
+		log.Fatal("Ox and legacy PGP can't be used simultaneously.")
 	case (*flagOx || *flagLegacyPGP) && *flagOOBFile != "":
 		log.Fatal("No encryption possible for OOB data.")
 	case (*flagOx || *flagLegacyPGP) && *flagHeadline:
@@ -213,7 +215,7 @@ func main() {
 		slog.Info("checking if OS user is root")
 		if currUser.Username == "root" {
 			fmt.Println("WARNING: It seems you are running go-sendxmpp as root user.\n" +
-				"This is is not recommended as go-sendxmpp does not require root " +
+				"This is not recommended as go-sendxmpp does not require root " +
 				"privileges. Please consider using a less privileged user. For an " +
 				"example how to do this with sudo please consult the manpage chapter " +
 				"TIPS.")
@@ -436,6 +438,10 @@ func main() {
 		NoSASLUpgrade:             *flagNoSASLUpgrade,
 		PeriodicServerPings:       psp,
 		PeriodicServerPingsPeriod: pspPeriod,
+		ReportSoftwareVersion:     true,
+		SoftwareName:              "go-sendxmpp",
+		SoftwareVersion:           version,
+		ReportSoftwareOS:          true,
 	}
 	slog.Info("setting xmpp connection option:", "Host", server)
 	slog.Info("setting xmpp connection option:", "User", user)
@@ -508,7 +514,13 @@ func main() {
 	client, err := connect(options, *flagDirectTLS, *flagAnon, *flagRetryConnect, *flagRetryConnectMax)
 	if err != nil {
 		if fast.Token != "" {
-			log.Fatal(fmt.Errorf("could not connect using FAST authentication: %v", err))
+			slog.Info("failed connecting using fast", "mechanism", options.FastMechanism, "token", options.FastToken)
+			options.FastToken = ""
+			options.FastMechanism = ""
+			client, err = connect(options, *flagDirectTLS, *flagAnon, *flagRetryConnect, *flagRetryConnectMax)
+			if err != nil {
+				log.Fatal(err)
+			}
 		} else {
 			log.Fatal(err)
 		}
@@ -553,9 +565,10 @@ func main() {
 	prsc := make(chan xmpp.Presence, defaultBufferSize)
 	disc := make(chan xmpp.DiscoItems, defaultBufferSize)
 	drc := make(chan xmpp.DiscoResult, defaultBufferSize)
+	slc := make(chan xmpp.Slot, defaultBufferSize)
 	ctx, cancel := context.WithCancel(context.Background())
 	slog.Info("starting to receive stanzas")
-	go rcvStanzas(client, ctx, iqc, msgc, prsc, disc, drc)
+	go rcvStanzas(client, ctx, iqc, msgc, prsc, disc, drc, slc)
 	for _, r := range recipientsList {
 		var re recipientsType
 		re.Jid = r
@@ -662,7 +675,7 @@ func main() {
 	var uploadMessages []string
 	if len(*flagHTTPUpload) != 0 && !*flagLegacyPGP {
 		slog.Info("http-upload:", "file", *flagHTTPUpload)
-		uploadMessages, err = httpUpload(client, iqc, disc, drc,
+		uploadMessages, err = httpUpload(client, iqc, disc, drc, slc,
 			jserver, *flagHTTPUpload, timeout, nil, nil)
 		if err != nil {
 			cancel()
@@ -939,10 +952,9 @@ func main() {
 			case len(*flagHTTPUpload) != 0 && !*flagLegacyPGP:
 				for _, message = range uploadMessages {
 					slog.Info("sending http-upload message:", "file", message, "JID", recipient.Jid)
-					_, err = client.Send(xmpp.Chat{
+					_, err = client.SendOOB(xmpp.Chat{
 						Remote: recipient.Jid,
-						Type:   msgType, Ooburl: message, Text: message,
-						Subject: *flagSubject,
+						Type:   msgType, Oob: xmpp.Oob{Url: message}, Subject: *flagSubject,
 					})
 					if err != nil {
 						fmt.Println("Couldn't send message to",
@@ -950,7 +962,7 @@ func main() {
 					}
 				}
 			case len(*flagHTTPUpload) != 0 && *flagLegacyPGP:
-				uploadMessages, err = httpUpload(client, iqc, disc, drc,
+				uploadMessages, err = httpUpload(client, iqc, disc, drc, slc,
 					jserver, *flagHTTPUpload, timeout, oxPrivKey, recipient.OxKeyRing)
 				if err != nil {
 					cancel()
@@ -958,10 +970,9 @@ func main() {
 				}
 				for _, message = range uploadMessages {
 					slog.Info("sending legacy pgp http-upload message:", "file", message, "JID", recipient.Jid)
-					_, err = client.Send(xmpp.Chat{
+					_, err = client.SendOOB(xmpp.Chat{
 						Remote: recipient.Jid,
-						Type:   msgType, Ooburl: message, Text: message,
-						Subject: *flagSubject,
+						Type:   msgType, Oob: xmpp.Oob{Url: message}, Subject: *flagSubject,
 					})
 					if err != nil {
 						fmt.Println("Couldn't send message to",
@@ -972,17 +983,10 @@ func main() {
 			// (Hopefully) temporary workaround due to go-xmpp choking on URL encoding.
 			// Once this is fixed in the lib the http-upload case above can be reused.
 			case *flagOOBFile != "":
-				var msg string
-				if *flagSubject != "" {
-					msg = fmt.Sprintf("<message to='%s' type='%s'><subject>%s</subject><body>%s</body><x xmlns='jabber:x:oob'><url>%s</url></x></message>",
-						recipient.Jid, msgType, *flagSubject, message, message)
-					slog.Info("sending OOB message:", "uri", message, "subject", *flagSubject, "JID", recipient.Jid)
-				} else {
-					msg = fmt.Sprintf("<message to='%s' type='%s'><body>%s</body><x xmlns='jabber:x:oob'><url>%s</url></x></message>",
-						recipient.Jid, msgType, message, message)
-					slog.Info("sending OOB message:", "uri", message, "JID", recipient.Jid)
-				}
-				_, err = client.SendOrg(msg)
+				_, err = client.SendOOB(xmpp.Chat{
+					Remote: recipient.Jid,
+					Type:   msgType, Oob: xmpp.Oob{Url: message}, Subject: *flagSubject,
+				})
 				if err != nil {
 					fmt.Println("Couldn't send message to",
 						recipient.Jid)
