@@ -12,7 +12,6 @@ import (
 	"io"
 	"log"
 	"log/slog"
-	"runtime"
 	"strings"
 	"time"
 
@@ -23,7 +22,7 @@ import (
 func getDiscoInfo(client *xmpp.Client, drc chan xmpp.DiscoResult, target string) (dr xmpp.DiscoResult, err error) {
 	id := getID()
 	c := make(chan xmpp.DiscoResult, defaultBufferSize)
-	go getDr(target, c, drc)
+	go getDr(c, drc)
 	slog.Info("stanzahandling: requesting disco info")
 	_, err = client.RawInformation(client.JID(), target, id, "get",
 		fmt.Sprintf("<query xmlns='%s'/>", nsDiscoInfo))
@@ -40,7 +39,7 @@ func getDiscoInfo(client *xmpp.Client, drc chan xmpp.DiscoResult, target string)
 	return dr, err
 }
 
-func getDr(jid string, c chan xmpp.DiscoResult, drc chan xmpp.DiscoResult) {
+func getDr(c chan xmpp.DiscoResult, drc chan xmpp.DiscoResult) {
 	for {
 		dr := <-drc
 		c <- dr
@@ -48,6 +47,33 @@ func getDr(jid string, c chan xmpp.DiscoResult, drc chan xmpp.DiscoResult) {
 	}
 }
 
+func getSlot(client *xmpp.Client, slc chan xmpp.Slot, target, request string) (sl xmpp.Slot, err error) {
+	id := getID()
+	c := make(chan xmpp.Slot, defaultBufferSize)
+	go getSl(c, slc)
+	slog.Info("stanzahandling: requesting upload slot")
+	_, err = client.RawInformation(client.JID(), target, id, "get", request)
+	if err != nil {
+		return sl, fmt.Errorf("get slot: failed to send iq: %w", err)
+	}
+	select {
+	case sl = <-c:
+		slog.Info("stanzahandling: requesting upload slot: received reply")
+	case <-time.After(60 * time.Second):
+		slog.Info("stanzahandling: requesting upload slot: timeout")
+		return sl, fmt.Errorf("get upload slot: %s didn't reply to upload slot request", target)
+	}
+	return sl, err
+}
+
+func getSl(c chan xmpp.Slot, slc chan xmpp.Slot) {
+	for {
+		sl := <-slc
+		c <- sl
+		return
+	}
+}
+
 func getDiscoItems(client *xmpp.Client, disc chan xmpp.DiscoItems, target string) (dis xmpp.DiscoItems, err error) {
 	id := getID()
 	c := make(chan xmpp.DiscoItems, defaultBufferSize)
@@ -116,7 +142,7 @@ func getPresence(from string, c chan xmpp.Presence, prsc chan xmpp.Presence) {
 	}
 }
 
-func rcvStanzas(client *xmpp.Client, ctx context.Context, iqc chan xmpp.IQ, msgc chan xmpp.Chat, prsc chan xmpp.Presence, disc chan xmpp.DiscoItems, drc chan xmpp.DiscoResult) {
+func rcvStanzas(client *xmpp.Client, ctx context.Context, iqc chan xmpp.IQ, msgc chan xmpp.Chat, prsc chan xmpp.Presence, disc chan xmpp.DiscoItems, drc chan xmpp.DiscoResult, slc chan xmpp.Slot) {
 	type messageError struct {
 		XMLName xml.Name `xml:"message-error"`
 		Text    string   `xml:"text"`
@@ -151,7 +177,7 @@ func rcvStanzas(client *xmpp.Client, ctx context.Context, iqc chan xmpp.IQ, msgc
 			case nil:
 				continue
 			default:
-				closeAndExit(client, fmt.Errorf("receive stanzas: %v", err))
+				closeAndExit(client, fmt.Errorf("receive stanzas: %w", err))
 				return
 			}
 		case received = <-r:
@@ -174,6 +200,8 @@ func rcvStanzas(client *xmpp.Client, ctx context.Context, iqc chan xmpp.IQ, msgc
 			disc <- v
 		case xmpp.DiscoResult:
 			drc <- v
+		case xmpp.Slot:
+			slc <- v
 		case xmpp.IQ:
 			switch v.Type {
 			case "get":
@@ -217,29 +245,6 @@ func rcvStanzas(client *xmpp.Client, ctx context.Context, iqc chan xmpp.IQ, msgc
 							log.Println(err)
 						}
 					}
-				case nsVersion:
-					root := etree.NewDocument()
-					root.WriteSettings.AttrSingleQuote = true
-					reply := root.CreateElement("iq")
-					reply.CreateAttr("type", "result")
-					reply.CreateAttr("from", client.JID())
-					reply.CreateAttr("to", v.From)
-					reply.CreateAttr("id", v.ID)
-					replyQuery := reply.CreateElement("query")
-					replyQuery.CreateAttr("xmlns", nsVersion)
-					rqName := replyQuery.CreateElement("name")
-					rqName.CreateText("go-sendxmpp")
-					rqVersion := replyQuery.CreateElement("version")
-					rqVersion.CreateText(version)
-					rqOS := replyQuery.CreateElement("os")
-					rqOS.CreateText(runtime.GOOS)
-					xmlString, err := root.WriteToString()
-					if err == nil {
-						_, err = client.SendOrg(xmlString)
-						if err != nil {
-							log.Println(err)
-						}
-					}
 				default:
 					root := etree.NewDocument()
 					root.WriteSettings.AttrSingleQuote = true
